@@ -14,10 +14,11 @@ use crate::util;
 use crate::util::{c_bool, Type, TypeContents};
 use opendp::traits::{MeasureDistance, MetricDistance};
 use std::fmt::Formatter;
+use std::hash::Hash;
 
 
 #[no_mangle]
-pub extern "C" fn opendp_data___slice_as_measure_distance(
+pub extern "C" fn opendp_data__slice_as_measure_distance(
     raw: *const FfiSlice, T: *const c_char
 ) -> FfiResult<*mut AnyMeasureDistance> {
     fn raw_to_plain<T: 'static + Clone + MeasureDistance>(raw: &FfiSlice) -> Fallible<AnyMeasureDistance> {
@@ -58,7 +59,7 @@ pub extern "C" fn opendp_data___slice_as_measure_distance(
 
 
 #[no_mangle]
-pub extern "C" fn opendp_data___slice_as_metric_distance(
+pub extern "C" fn opendp_data__slice_as_metric_distance(
     raw: *const FfiSlice, T: *const c_char
 ) -> FfiResult<*mut AnyMetricDistance> {
     fn raw_to_plain<T: 'static + Clone + MetricDistance>(raw: &FfiSlice) -> Fallible<AnyMetricDistance> {
@@ -78,7 +79,9 @@ pub extern "C" fn opendp_data___slice_as_metric_distance(
 }
 
 #[no_mangle]
-pub extern "C" fn opendp_data___slice_as_object(raw: *const FfiSlice, T: *const c_char) -> FfiResult<*mut AnyObject> {
+pub extern "C" fn opendp_data__slice_as_object(raw: *const FfiSlice, T: *const c_char) -> FfiResult<*mut AnyObject> {
+    let raw = try_as_ref!(raw);
+    let T = try_!(Type::try_from(T));
     fn raw_to_plain<T: 'static + Clone>(raw: &FfiSlice) -> Fallible<AnyObject> {
         if raw.len != 1 {
             return fallible!(FFI, "The slice length must be one when creating a scalar from FfiSlice");
@@ -91,6 +94,13 @@ pub extern "C" fn opendp_data___slice_as_object(raw: *const FfiSlice, T: *const
         let string = util::to_str(raw.ptr as *const c_char)?.to_owned();
         Ok(AnyObject::new(string))
     }
+    fn raw_to_vec_string(raw: &FfiSlice) -> Fallible<AnyObject> {
+        let slice = unsafe { slice::from_raw_parts(raw.ptr as *const *const c_char, raw.len) };
+        let vec = slice.iter()
+            .map(|str_ptr| Ok(util::to_str(*str_ptr)?.to_owned()))
+            .collect::<Fallible<Vec<String>>>()?;
+        Ok(AnyObject::new(vec))
+    }
     fn raw_to_slice<T: Clone>(_raw: &FfiSlice) -> Fallible<AnyObject> {
         // TODO: Need to do some extra wrapping to own the slice here.
         unimplemented!()
@@ -112,9 +122,22 @@ pub extern "C" fn opendp_data___slice_as_object(raw: *const FfiSlice, T: *const
             .ok_or_else(|| err!(FFI, "Attempted to follow a null pointer to create a tuple"))?;
         Ok(AnyObject::new(tuple))
     }
-    let T = try_!(Type::try_from(T));
-    let raw = try_as_ref!(raw);
-    let obj = match T.contents {
+    fn raw_to_hashmap<K: 'static + Clone + Hash + Eq, V: 'static + Clone>(raw: &FfiSlice) -> Fallible<AnyObject> {
+        let slice = unsafe { slice::from_raw_parts(raw.ptr as *const *const AnyObject, raw.len) };
+
+        // unpack keys and values into slices
+        if slice.len() != 2 {return fallible!(FFI, "HashMap FfiSlice must have length 2")}
+        let keys = try_as_ref!(slice[0]).downcast_ref::<Vec<K>>()?;
+        let vals = try_as_ref!(slice[1]).downcast_ref::<Vec<V>>()?;
+
+        // construct the hashmap
+        if keys.len() != vals.len() {return fallible!(FFI, "HashMap FfiSlice must have an equivalent number of keys and values")};
+        let map = keys.iter().cloned()
+            .zip(vals.iter().cloned())
+            .collect::<HashMap<K, V>>();
+        Ok(AnyObject::new(map))
+    }
+    match T.contents {
         TypeContents::PLAIN("String") => {
             raw_to_string(raw)
         }
@@ -124,7 +147,11 @@ pub extern "C" fn opendp_data___slice_as_object(raw: *const FfiSlice, T: *const
         }
         TypeContents::VEC(element_id) => {
             let element = try_!(Type::of_id(&element_id));
-            dispatch!(raw_to_vec, [(element, @primitives)], (raw))
+            if element.descriptor == "String" {
+                raw_to_vec_string(raw)
+            } else {
+                dispatch!(raw_to_vec, [(element, @primitives)], (raw))
+            }
         }
         TypeContents::TUPLE(ref element_ids) => {
             if element_ids.len() != 2 {
@@ -135,13 +162,20 @@ pub extern "C" fn opendp_data___slice_as_object(raw: *const FfiSlice, T: *const
             // because the only likely way to get a tuple of AnyObjects is as the output of composition.
             dispatch!(raw_to_tuple, [(types[0], @primitives), (types[1], @primitives)], (raw))
         },
+        TypeContents::GENERIC { name, args } => {
+            if name == "HashMap" {
+                if args.len() != 2 {return err!(FFI, "HashMaps should have 2 type arguments").into()}
+                let K = try_!(Type::of_id(&args[0]));
+                let V = try_!(Type::of_id(&args[1]));
+                dispatch!(raw_to_hashmap, [(K, @hashable), (V, @primitives)], (raw))
+            } else {fallible!(FFI, "unrecognized generic {:?}", name)}
+        }
         _ => dispatch!(raw_to_plain, [(T, @primitives)], (raw))
-    };
-    obj.into()
+    }.into()
 }
 
 #[no_mangle]
-pub extern "C" fn opendp_data___object_type(this: *mut AnyObject) -> FfiResult<*mut c_char> {
+pub extern "C" fn opendp_data__object_type(this: *mut AnyObject) -> FfiResult<*mut c_char> {
     let obj: &AnyObject = try_as_ref!(this);
 
     match util::into_c_char_p(obj.type_.descriptor.to_string()) {
@@ -151,7 +185,8 @@ pub extern "C" fn opendp_data___object_type(this: *mut AnyObject) -> FfiResult<*
 }
 
 #[no_mangle]
-pub extern "C" fn opendp_data___object_as_slice(obj: *const AnyObject) -> FfiResult<*mut FfiSlice> {
+pub extern "C" fn opendp_data__object_as_slice(obj: *const AnyObject) -> FfiResult<*mut FfiSlice> {
+    let obj = try_as_ref!(obj);
     fn plain_to_raw<T: 'static>(obj: &AnyObject) -> Fallible<FfiSlice> {
         let plain: &T = obj.downcast_ref()?;
         Ok(FfiSlice::new(plain as *const T as *mut c_void, 1))
@@ -161,6 +196,16 @@ pub extern "C" fn opendp_data___object_as_slice(obj: *const AnyObject) -> FfiRes
         // FIXME: There's no way to get a CString without copying, so this leaks.
         Ok(FfiSlice::new(util::into_c_char_p(string.clone())? as *mut c_void, string.len() + 1))
     }
+    fn vec_string_to_raw(obj: &AnyObject) -> Fallible<FfiSlice> {
+        let vec_str: &Vec<String> = obj.downcast_ref()?;
+        let vec = vec_str.iter()
+            .cloned().map(util::into_c_char_p)
+            .collect::<Fallible<Vec<*mut c_char>>>()?;
+
+        let res = Ok(FfiSlice::new(vec.as_ptr() as *mut c_void, vec.len()));
+        util::into_raw(vec);
+        res
+    }
     fn slice_to_raw<T>(_obj: &AnyObject) -> Fallible<FfiSlice> {
         // TODO: Need to get a reference to the slice here.
         unimplemented!()
@@ -176,8 +221,20 @@ pub extern "C" fn opendp_data___object_as_slice(obj: *const AnyObject) -> FfiRes
             &tuple.1 as *const T1 as *const c_void
         ]) as *mut c_void, 2))
     }
-    let obj = try_as_ref!(obj);
-    let raw = match &obj.type_.contents {
+    fn hashmap_to_raw<K: 'static + Clone + Hash + Eq, V: 'static + Clone>(obj: &AnyObject) -> Fallible<FfiSlice> {
+        let data: &HashMap<K, V> = obj.downcast_ref()?;
+
+        // wrap keys and values up in an AnyObject
+        let keys = AnyObject::new(data.keys().cloned().collect::<Vec<K>>());
+        let vals = AnyObject::new(data.values().cloned().collect::<Vec<V>>());
+
+        // wrap the whole map up together in an FfiSlice
+        let map = vec![util::into_raw(keys), util::into_raw(vals)];
+        let map_slice = FfiSlice::new(map.as_ptr() as *mut c_void, map.len());
+        util::into_raw(map);
+        Ok(map_slice)
+    }
+    match &obj.type_.contents {
         TypeContents::PLAIN("String") => {
             string_to_raw(obj)
         }
@@ -187,7 +244,11 @@ pub extern "C" fn opendp_data___object_as_slice(obj: *const AnyObject) -> FfiRes
         }
         TypeContents::VEC(element_id) => {
             let element = try_!(Type::of_id(element_id));
-            dispatch!(vec_to_raw, [(element, @primitives)], (obj))
+            if element.descriptor == "String" {
+                vec_string_to_raw(obj)
+            } else {
+                dispatch!(vec_to_raw, [(element, @primitives)], (obj))
+            }
         }
         TypeContents::TUPLE(element_ids) => {
             if element_ids.len() != 2 {
@@ -197,29 +258,36 @@ pub extern "C" fn opendp_data___object_as_slice(obj: *const AnyObject) -> FfiRes
             // In the outbound direction, we can handle tuples of both primitives and AnyObjects.
             dispatch!(tuple_to_raw, [(types[0], @primitives_plus), (types[1], @primitives_plus)], (obj))
         }
+        TypeContents::GENERIC { name, args } => {
+            if name == &"HashMap" {
+                if args.len() != 2 {return err!(FFI, "HashMaps should have 2 type arguments").into()}
+                let K = try_!(Type::of_id(&args[0]));
+                let V = try_!(Type::of_id(&args[1]));
+                dispatch!(hashmap_to_raw, [(K, @hashable), (V, @primitives)], (obj))
+            } else {fallible!(FFI, "unrecognized generic {:?}", name)}
+        }
         _ => { dispatch!(plain_to_raw, [(&obj.type_, @primitives)], (obj)) }
-    };
-    raw.into()
+    }.into()
 }
 
 #[no_mangle]
-pub extern "C" fn opendp_data___object_free(this: *mut AnyObject) -> FfiResult<*mut ()> {
+pub extern "C" fn opendp_data__object_free(this: *mut AnyObject) -> FfiResult<*mut ()> {
     util::into_owned(this).map(|_| ()).into()
 }
 
 #[no_mangle]
 /// Frees the slice, but not what the slice references!
-pub extern "C" fn opendp_data___slice_free(this: *mut FfiSlice) -> FfiResult<*mut ()> {
+pub extern "C" fn opendp_data__slice_free(this: *mut FfiSlice) -> FfiResult<*mut ()> {
     util::into_owned(this).map(|_| ()).into()
 }
 
 #[no_mangle]
-pub extern "C" fn opendp_data___str_free(this: *mut c_char) -> FfiResult<*mut ()> {
+pub extern "C" fn opendp_data__str_free(this: *mut c_char) -> FfiResult<*mut ()> {
     util::into_owned(this).map(|_| ()).into()
 }
 
 #[no_mangle]
-pub extern "C" fn opendp_data___bool_free(this: *mut c_bool) -> FfiResult<*mut ()> {
+pub extern "C" fn opendp_data__bool_free(this: *mut c_bool) -> FfiResult<*mut ()> {
     util::into_owned(this).map(|_| ()).into()
 }
 
@@ -248,7 +316,7 @@ impl std::fmt::Debug for AnyObject {
 }
 
 #[no_mangle]
-pub extern "C" fn opendp_data___to_string(this: *const AnyObject) -> FfiResult<*mut c_char> {
+pub extern "C" fn opendp_data__to_string(this: *const AnyObject) -> FfiResult<*mut c_char> {
     util::into_c_char_p(format!("{:?}", try_as_ref!(this))).map_or_else(
         |e| FfiResult::Err(util::into_raw(FfiError::from(e))),
         FfiResult::Ok)
@@ -268,7 +336,7 @@ mod tests {
         let raw_ptr = util::into_raw(999) as *mut c_void;
         let raw_len = 1;
         let raw = util::into_raw(FfiSlice::new(raw_ptr, raw_len));
-        let res = opendp_data___slice_as_object(raw, "i32".to_char_p());
+        let res = opendp_data__slice_as_object(raw, "i32".to_char_p());
         let res: i32 = Fallible::from(res)?.downcast()?;
         assert_eq!(res, 999);
         Ok(())
@@ -280,7 +348,7 @@ mod tests {
         let raw_ptr = util::into_c_char_p(data.clone()).unwrap_test() as *mut c_void;
         let raw_len = data.len() + 1;
         let raw = util::into_raw(FfiSlice::new(raw_ptr, raw_len));
-        let res = opendp_data___slice_as_object(raw, "String".to_char_p());
+        let res = opendp_data__slice_as_object(raw, "String".to_char_p());
         let res: String = Fallible::from(res)?.downcast()?;
         assert_eq!(res, "Hello");
         Ok(())
@@ -292,7 +360,7 @@ mod tests {
         let raw_ptr = data.as_ptr() as *mut c_void;
         let raw_len = data.len();
         let raw = util::into_raw(FfiSlice::new(raw_ptr, raw_len));
-        let res = opendp_data___slice_as_object(raw, "Vec<i32>".to_char_p());
+        let res = opendp_data__slice_as_object(raw, "Vec<i32>".to_char_p());
         let res: Vec<i32> = Fallible::from(res)?.downcast()?;
         assert_eq!(res, vec![1, 2, 3]);
         Ok(())
@@ -303,7 +371,7 @@ mod tests {
         let raw_ptr = util::into_raw((util::into_raw(999), util::into_raw(-999))) as *mut c_void;
         let raw_len = 2;
         let raw = util::into_raw(FfiSlice::new(raw_ptr, raw_len));
-        let res = opendp_data___slice_as_object(raw, "(i32, i32)".to_char_p());
+        let res = opendp_data__slice_as_object(raw, "(i32, i32)".to_char_p());
         let res: (i32, i32) = Fallible::from(res)?.downcast()?;
         assert_eq!(res, (999, -999));
         Ok(())
@@ -312,7 +380,7 @@ mod tests {
     #[test]
     fn test_data_as_raw_number() -> Fallible<()> {
         let obj = AnyObject::new_raw(999);
-        let res = opendp_data___object_as_slice(obj);
+        let res = opendp_data__object_as_slice(obj);
         let res = Fallible::from(res)?;
         assert_eq!(res.len, 1);
         assert_eq!(util::as_ref(res.ptr as *const i32).unwrap_test(), &999);
@@ -322,7 +390,7 @@ mod tests {
     #[test]
     fn test_data_as_raw_string() -> Fallible<()> {
         let obj = AnyObject::new_raw("Hello".to_owned());
-        let res = opendp_data___object_as_slice(obj);
+        let res = opendp_data__object_as_slice(obj);
         let res = Fallible::from(res)?;
         assert_eq!(res.len, 6);
         assert_eq!(util::into_string(res.ptr as *mut c_char).unwrap_test(), "Hello");
@@ -332,7 +400,7 @@ mod tests {
     #[test]
     fn test_data_as_raw_vec() -> Fallible<()> {
         let obj = AnyObject::new_raw(vec![1, 2, 3]);
-        let res = opendp_data___object_as_slice(obj);
+        let res = opendp_data__object_as_slice(obj);
         let res = Fallible::from(res)?;
         assert_eq!(res.len, 3);
         assert_eq!(util::as_ref(res.ptr as *const [i32;3]).unwrap_test(), &[1, 2, 3]);
@@ -342,7 +410,7 @@ mod tests {
     #[test]
     fn test_data_as_raw_tuple_numbers() -> Fallible<()> {
         let obj = AnyObject::new_raw((999, -999));
-        let res = opendp_data___object_as_slice(obj);
+        let res = opendp_data__object_as_slice(obj);
         let res = Fallible::from(res)?;
         assert_eq!(res.len, 2);
         let res_ptr = util::as_ref(res.ptr as *const [*mut i32;2]).unwrap_test();
@@ -353,7 +421,7 @@ mod tests {
     #[test]
     fn test_data_as_raw_tuple_objects() -> Fallible<()> {
         let obj = AnyObject::new_raw((AnyObject::new(999), AnyObject::new(999.0)));
-        let res = opendp_data___object_as_slice(obj);
+        let res = opendp_data__object_as_slice(obj);
         let res = Fallible::from(res)?;
         assert_eq!(res.len, 2);
         let res_ptr = util::as_ref(res.ptr as *const [*mut AnyObject;2]).unwrap_test();
