@@ -1,35 +1,34 @@
 use std::convert::TryFrom;
 use std::iter::Sum;
 use std::ops::Sub;
-use std::os::raw::{c_char, c_uint, c_void};
+use std::os::raw::{c_char, c_uint};
 
 use num::Float;
 
 use opendp::err;
-use opendp::traits::{DistanceConstant, InfCast, ExactIntCast, CheckedMul};
-use opendp::trans::{make_bounded_mean};
+use opendp::traits::{DistanceConstant, InfCast, ExactIntCast, CheckedMul, CheckNull};
+use opendp::trans::{make_sized_bounded_mean};
 
-use crate::any::AnyTransformation;
+use crate::any::{AnyTransformation, AnyObject, Downcast};
 use crate::core::{FfiResult, IntoAnyTransformationFfiResultExt};
 use crate::util::Type;
 use opendp::dist::IntDistance;
 
 #[no_mangle]
-pub extern "C" fn opendp_trans__make_bounded_mean(
-    lower: *const c_void, upper: *const c_void, n: c_uint,
+pub extern "C" fn opendp_trans__make_sized_bounded_mean(
+    size: c_uint, bounds: *const AnyObject,
     T: *const c_char,
 ) -> FfiResult<*mut AnyTransformation> {
-    fn monomorphize<T>(lower: *const c_void, upper: *const c_void, n: usize) -> FfiResult<*mut AnyTransformation>
-        where T: DistanceConstant<IntDistance> + Sub<Output=T> + Float + ExactIntCast<usize> + CheckedMul,
+    fn monomorphize<T>(size: usize, bounds: *const AnyObject) -> FfiResult<*mut AnyTransformation>
+        where T: DistanceConstant<IntDistance> + Sub<Output=T> + Float + ExactIntCast<usize> + CheckedMul + CheckNull,
               for<'a> T: Sum<&'a T>,
               IntDistance: InfCast<T> {
-        let lower = *try_as_ref!(lower as *const T);
-        let upper = *try_as_ref!(upper as *const T);
-        make_bounded_mean::<T>(lower, upper, n).into_any()
+        let bounds = try_!(try_as_ref!(bounds).downcast_ref::<(T, T)>()).clone();
+        make_sized_bounded_mean::<T>(size, bounds).into_any()
     }
-    let n = n as usize;
+    let size = size as usize;
     let T = try_!(Type::try_from(T));
-    dispatch!(monomorphize, [(T, @floats)], (lower, upper, n))
+    dispatch!(monomorphize, [(T, @floats)], (size, bounds))
 }
 
 
@@ -46,10 +45,9 @@ mod tests {
 
     #[test]
     fn test_make_bounded_sum_n() -> Fallible<()> {
-        let transformation = Result::from(opendp_trans__make_bounded_mean(
-            util::into_raw(0.0) as *const c_void,
-            util::into_raw(10.0) as *const c_void,
+        let transformation = Result::from(opendp_trans__make_sized_bounded_mean(
             3 as c_uint,
+            util::into_raw(AnyObject::new((0., 10.))),
             "f64".to_char_p(),
         ))?;
         let arg = AnyObject::new_raw(vec![1.0, 2.0, 3.0]);
